/** @module player/state */
import { put, takeEvery } from 'redux-saga/effects'
import { generatePlayerList } from './functions'

/**
 * state.js
 *
 * Collection of everything which has to do with state changes.
 **/

/** actionTypes define what actions are handeled by the reducer. */
export const actions = {
  setState: players => {
    return {
      type: 'SET_PLAYERS',
      players
    }
  },
  fileUploadStart: event => {
    return {
      type: 'PLAYER_FILE_UPLOAD_START',
      event
    }
  },
  fileUploadSuccess: () => {
    return {
      type: 'PLAYER_FILE_UPLOAD_SUCCESS'
    }
  },
  fileUploadFailure: () => {
    return {
      type: 'PLAYER_FILE_UPLOAD_FAILURE'
    }
  }
}
console.log('State actions', actions)

/** state definition */
export const state = {
  allPlayers: [],
  filteredPlayers: [],
  filters: {},
  fileUpload: 'idle'
}
console.log('State state', state)

/** reducer is called by the redux dispatcher and handles all component actions */
export function reducer (state = [], action) {
  switch (action.type) {
    case 'SET_PLAYERS':
      return { ...state, allPlayers: action.players }
    case 'PLAYER_FILE_UPLOAD_START':
      return { ...state, fileUpload: 'started' }
    case 'PLAYER_FILE_UPLOAD_SUCCESS':
      return { ...state, fileUpload: 'finished' }
    case 'PLAYER_FILE_UPLOAD_FAILURE':
      return { ...state, fileUpload: 'failure' }
    default:
      return state
  }
}

function * uploadFile (action) {
  console.log('PlayerList uploadFile')
  try {
    console.log(action.event)
    generatePlayerList(action)
    yield put(actions.fileUploadSuccess())
  } catch (e) {
    yield put(actions.fileUploadFailure())
  }
}

/** sagas are asynchronous workers (JS generators) to handle the state. */
export function * saga () {
  console.log('Player saga started.')
  yield takeEvery('PLAYER_FILE_UPLOAD_START', uploadFile)
}
